
local StoneGateV6 = ...

-- Apply building asset processor
StoneGateV6:registerAssetProcessor("models/basic/basicSmallCircularTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Small Circular Tower materials
StoneGateV6:registerAssetId("models/basic/basicSmallCircularTower.fbx/Materials/redRoof", StoneGateSetPrefix .. "_SCT_RED_ROOF_MATERIAL")
StoneGateV6:registerAssetId("models/basic/basicSmallCircularTower.fbx/Materials/blueRoof", StoneGateSetPrefix .. "_SCT_BLUE_ROOF_MATERIAL")
StoneGateV6:registerAssetId("models/basic/basicSmallCircularTower.fbx/Materials/brownRoof", StoneGateSetPrefix .. "_SCT_BROWN_ROOF_MATERIAL")
StoneGateV6:registerAssetId("models/basic/basicSmallCircularTower.fbx/Materials/greyRoof", StoneGateSetPrefix .. "_SCT_GREY_ROOF_MATERIAL")
StoneGateV6:registerAssetId("models/basic/basicSmallCircularTower.fbx/Materials/grey2Roof", StoneGateSetPrefix .. "_SCT_GREY2_ROOF_MATERIAL")

-- Register Small Circular Tower prefab nodes
StoneGateV6:registerAssetId("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_0_Sct_RootPart", StoneGateSetPrefix .. "_SCT_ROOT_PREFAB")
StoneGateV6:registerAssetId("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_1_Sct_Tiling1Part", StoneGateSetPrefix .. "_SCT_TILING1_PREFAB")
StoneGateV6:registerAssetId("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_2_Sct_Tiling2Part", StoneGateSetPrefix .. "_SCT_TILING2_PREFAB")
StoneGateV6:registerAssetId("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_1_Sct_Tiling1PPart", StoneGateSetPrefix .. "_SCT_TILING1P_PREFAB")
StoneGateV6:registerAssetId("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_2_Sct_Tiling2PPart", StoneGateSetPrefix .. "_SCT_TILING2P_PREFAB")
StoneGateV6:registerAssetId("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_3_Sct_TopPart", StoneGateSetPrefix .. "_SCT_TOP_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart2(_nodePrefix)
	StoneGateV6:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	        }
	})
end

local defaultNodePrefixList2 = {
	StoneGateSetPrefix .. "_SCT_TOP",
	StoneGateSetPrefix .. "_SCT_TILING1",
	StoneGateSetPrefix .. "_SCT_TILING2",
	StoneGateSetPrefix .. "_SCT_TILING1P",
	StoneGateSetPrefix .. "_SCT_TILING2P"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList2) do
	registerDefaultBuildingPart2(nodePrefix)
end

-- Register the stone gate set small circular tower root part
StoneGateV6:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SCT_ROOT_PART",
	Name = StoneGateSetPrefix .. "_SCT_ROOT_NAME",
	Description = StoneGateSetPrefix .. "_SCT_ROOT_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SCT_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SCT_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SCT_TILING2_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SCT_TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true,
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 2,
	AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_LORD_MANOR"
})

-- Register the stone gate set circular tower root part
StoneGateV6:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SCT_ROOTP_PART",
	Name = StoneGateSetPrefix .. "_SCT_ROOTP_NAME",
	Description = StoneGateSetPrefix .. "_SCT_ROOTP_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SCT_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SCT_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SCT_TILING2P_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SCT_TILING1P_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true,
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "WOOD", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 2,
	AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_LORD_MANOR"
})

--------------------------------------------------------------------------

-- Register flag assigner component on the collider's type to 'PLATFORM'

--StoneGateV6:configurePrefabFlagList("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_3_Sct_TopPart", { "PLATFORM" })
 
-- Register grass clearing components

StoneGateV6:registerPrefabComponent("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_0_Sct_RootPart", {
	DataType = "COMP_DIRT_CIRCLE",
	HardRadius = 1,
        SoftRadius = 1,
        IsPermanent = true
})

-- Override building part components properties (generated by the building asset processor) to add building zones
StoneGateV6:registerPrefabComponent("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/step_0_Sct_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZoneData = {
            DataType = "BUILDING_ZONE", 
            Polygon = polygon.createCircle( 1, { 0, 0 }, 24 )
        }
})

function registerRoofPart(_nameFbx, _nameAsset)
    StoneGateV6:registerAssetId("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGateV6:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5 }
		}
	}
    })

    StoneGateV6:registerPrefabComponent("models/basic/basicSmallCircularTower.fbx/Prefab/smallCircularTower/" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 0, 0 } -- For future snapping over large gate
    })
end

registerRoofPart( "SmallCircularRoof", "ROOF")
registerRoofPart( "SmallCircularRoofB", "ROOFB")

---- Ramparts...

-- Apply building asset processor
StoneGateV6:registerAssetProcessor("models/ramparts/rampartSmallCircularTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

function registerRampartPart(_nameFbx, _nameAsset)
    StoneGateV6:registerAssetId("models/ramparts/rampartSmallCircularTower.fbx/Prefab/smallCircularTower/step_4_Sct_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGateV6:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_SCT_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	}
    })

    StoneGateV6:registerPrefabComponent("models/ramparts/rampartSmallCircularTower.fbx/Prefab/smallCircularTower/step_4_Sct_" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
--	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 0, 0 } -- For future snapping over large gate
    })
end

registerRampartPart( "RampartA", "RAMPART_A" )
registerRampartPart( "RampartB", "RAMPART_B" )
